@@ -19,9 +19,10 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#include "mojito-view-bindings.h"
 #include "mojito-client-view.h"
-#include "mojito-client-marshals.h"
+
+#include <interfaces/mojito-view-bindings.h>
+#include <interfaces/mojito-marshals.h>
 
 G_DEFINE_TYPE (MojitoClientView, mojito_client_view, G_TYPE_OBJECT)
 
@@ -31,10 +32,10 @@ G_DEFINE_TYPE (MojitoClientView, mojito_client_view, G_TYPE_OBJECT)
 typedef struct _MojitoClientViewPrivate MojitoClientViewPrivate;
 
 struct _MojitoClientViewPrivate {
-    gchar *object_path;
-    DBusGConnection *connection;
-    DBusGProxy *proxy;
-    GHashTable *uuid_to_items;
+  gchar *object_path;
+  DBusGConnection *connection;
+  DBusGProxy *proxy;
+  GHashTable *uuid_to_items;
 };
 
 enum
@@ -45,9 +46,10 @@ enum
 
 enum
 {
-  ITEM_ADDED_SIGNAL,
-  ITEM_CHANGED_SIGNAL,
-  ITEM_REMOVED_SIGNAL,
+  ITEMS_ADDED_SIGNAL,
+  ITEMS_CHANGED_SIGNAL,
+  ITEMS_REMOVED_SIGNAL,
+
   LAST_SIGNAL
 };
 
@@ -123,75 +125,175 @@ mojito_client_view_finalize (GObject *object)
 }
 
 static void
-_proxy_item_added_cb (DBusGProxy  *proxy,
-                      const gchar *service,
-                      const gchar *uuid,
-                      gint64       date,
-                      GHashTable  *props,
-                      gpointer     userdata)
+_mojito_item_update_from_value_array (MojitoItem  *item,
+                                      GValueArray *varray)
+{
+  if (item->service)
+  {
+    g_free (item->service);
+    item->service = NULL;
+  }
+
+  item->service = g_value_dup_string (g_value_array_get_nth (varray, 0));
+
+  if (item->uuid)
+  {
+    g_free (item->uuid);
+    item->uuid = NULL;
+  }
+
+  item->uuid = g_value_dup_string (g_value_array_get_nth (varray, 1));
+
+  item->date.tv_sec = g_value_get_int64 (g_value_array_get_nth (varray, 2));
+
+  if (item->props)
+  {
+    g_hash_table_unref (item->props);
+    item->props = NULL;
+  }
+
+  item->props = g_value_dup_boxed (g_value_array_get_nth (varray, 3));
+}
+
+static MojitoItem *
+_mojito_item_from_value_array (GValueArray *varray)
 {
-  MojitoClientView *view = MOJITO_CLIENT_VIEW (userdata);
-  MojitoClientViewPrivate *priv = GET_PRIVATE (userdata);
   MojitoItem *item;
 
   item = mojito_item_new ();
-  item->service = g_strdup (service);
-  item->uuid = g_strdup (uuid);
-  item->date.tv_sec = date;
-  item->props = g_hash_table_ref (props);
+  _mojito_item_update_from_value_array (item, varray);
 
-  g_hash_table_insert (priv->uuid_to_items, g_strdup (uuid), item);
-
-  g_signal_emit (view, signals[ITEM_ADDED_SIGNAL], 0, item);
+  return item;
 }
 
 static void
-_proxy_item_changed_cb (DBusGProxy  *proxy,
-                        const gchar *service,
-                        const gchar *uuid,
-                        gint64       date,
-                        GHashTable  *props,
-                        gpointer     userdata)
+_proxy_items_added_cb (DBusGProxy *proxy,
+                       GPtrArray  *items,
+                       gpointer    userdata)
 {
   MojitoClientView *view = MOJITO_CLIENT_VIEW (userdata);
-  MojitoClientViewPrivate *priv = GET_PRIVATE (userdata);
-  MojitoItem *item = NULL;
+  MojitoClientViewPrivate *priv = GET_PRIVATE (view);
+  gint i = 0;
+  GList *items_list = NULL;
 
-  item = g_hash_table_lookup (priv->uuid_to_items, uuid);
-
-  if (item)
+  for (i = 0; i < items->len; i++)
   {
-    g_hash_table_unref (item->props);
-    item->props = g_hash_table_ref (props);
-    g_signal_emit (view, signals[ITEM_ADDED_SIGNAL], 0, item);
-  } else {
-    g_critical (G_STRLOC ": Unknown changed item with uuid: %s.",
-                uuid);
+    GValueArray *varray = (GValueArray *)g_ptr_array_index (items, i);
+    MojitoItem *item;
+
+    /* First reference dropped when list freed */
+    item = _mojito_item_from_value_array (varray);
+
+    g_hash_table_insert (priv->uuid_to_items,
+                         g_strdup (item->uuid),
+                         mojito_item_ref (item));
+
+    items_list = g_list_append (items_list, item);
   }
+
+  /* If handler wants a ref then it should ref it up */
+  g_signal_emit (view, signals[ITEMS_ADDED_SIGNAL], 0, items_list);
+
+  g_list_foreach (items_list, (GFunc)mojito_item_unref, NULL);
+  g_list_free (items_list);
 }
 
 static void
-_proxy_item_removed_cb (DBusGProxy  *proxy,
-                        const gchar *service,
-                        const gchar *uuid,
-                        gpointer     userdata)
+_proxy_items_changed_cb (DBusGProxy *proxy,
+                         GPtrArray  *items,
+                         gpointer    userdata)
 {
   MojitoClientView *view = MOJITO_CLIENT_VIEW (userdata);
-  MojitoClientViewPrivate *priv = GET_PRIVATE (userdata);
-  MojitoItem *item = NULL;
+  MojitoClientViewPrivate *priv = GET_PRIVATE (view);
+  gint i = 0;
+  GList *items_list = NULL;
 
-  item = g_hash_table_lookup (priv->uuid_to_items, uuid);
-
-  if (item)
+  for (i = 0; i < items->len; i++)
   {
-    mojito_item_ref (item);
-    g_hash_table_remove (priv->uuid_to_items, uuid);
-    g_signal_emit (view, signals[ITEM_REMOVED_SIGNAL], 0, item);
-    mojito_item_unref (item);
-  } else {
-    g_critical (G_STRLOC ": Asked to remove unknown item with uuid: %s",
-                uuid);
+    GValueArray *varray = (GValueArray *)g_ptr_array_index (items, i);
+    MojitoItem *item;
+    const gchar *uid;
+
+    uid = g_value_get_string (g_value_array_get_nth (varray, 1));
+
+    item = g_hash_table_lookup (priv->uuid_to_items,
+                                uid);
+    _mojito_item_update_from_value_array (item, varray);
+
+    items_list = g_list_append (items_list, mojito_item_ref (item));
   }
+
+  /* If handler wants a ref then it should ref it up */
+  g_signal_emit (view, signals[ITEMS_CHANGED_SIGNAL], 0, items_list);
+
+  g_list_foreach (items_list, (GFunc)mojito_item_unref, NULL);
+  g_list_free (items_list);
+}
+
+static void
+_proxy_items_removed_cb (DBusGProxy *proxy,
+                         GPtrArray  *items,
+                         gpointer    userdata)
+{
+  MojitoClientView *view = MOJITO_CLIENT_VIEW (userdata);
+  MojitoClientViewPrivate *priv = GET_PRIVATE (view);
+  gint i = 0;
+  GList *items_list = NULL;
+
+  for (i = 0; i < items->len; i++)
+  {
+    GValueArray *varray = (GValueArray *)g_ptr_array_index (items, i);
+    const gchar *uid;
+    MojitoItem *item;
+
+    uid = g_value_get_string (g_value_array_get_nth (varray, 1));
+
+    item = g_hash_table_lookup (priv->uuid_to_items,
+                                uid);
+
+    if (item)
+    {
+      /* Must ref up because g_hash_table_remove drops ref */
+      items_list = g_list_append (items_list, mojito_item_ref (item));
+      g_hash_table_remove (priv->uuid_to_items, uid);
+    }
+  }
+
+  /* If handler wants a ref then it should ref it up */
+  g_signal_emit (view, signals[ITEMS_REMOVED_SIGNAL], 0, items_list);
+
+  g_list_foreach (items_list, (GFunc)mojito_item_unref, NULL);
+  g_list_free (items_list);
+}
+
+static GType
+_mojito_item_get_struct_type (void)
+{
+  return dbus_g_type_get_struct ("GValueArray",
+                                 G_TYPE_STRING,
+                                 G_TYPE_STRING,
+                                 G_TYPE_INT64,
+                                 dbus_g_type_get_map ("GHashTable",
+                                                      G_TYPE_STRING,
+                                                      G_TYPE_STRING),
+                                 G_TYPE_INVALID);
+}
+
+static GType
+_mojito_items_get_container_type (void)
+{
+ return dbus_g_type_get_collection ("GPtrArray",
+                                    _mojito_item_get_struct_type ());
+}
+
+static GType
+_mojito_items_removed_get_container_type (void)
+{
+  return dbus_g_type_get_collection ("GPtrArray",
+                                     dbus_g_type_get_struct ("GValueArray",
+                                                             G_TYPE_STRING,
+                                                             G_TYPE_STRING,
+                                                             G_TYPE_INVALID));
 }
 
 static void
@@ -240,53 +342,33 @@ mojito_client_view_constructed (GObject *object)
     return;
   }
 
-  dbus_g_object_register_marshaller (mojito_marshal_VOID__STRING_STRING_INT64_BOXED,
-                                     G_TYPE_NONE,
-                                     G_TYPE_STRING,
-                                     G_TYPE_STRING,
-                                     G_TYPE_INT64,
-                                     DBUS_TYPE_G_STRING_STRING_HASHTABLE,
-                                     G_TYPE_INVALID);
-  dbus_g_object_register_marshaller (mojito_marshal_VOID__STRING_STRING,
-                                     G_TYPE_NONE,
-                                     G_TYPE_STRING,
-                                     G_TYPE_STRING,
-                                     G_TYPE_INVALID);
-
   dbus_g_proxy_add_signal (priv->proxy,
-                           "ItemAdded",
-                           G_TYPE_STRING,
-                           G_TYPE_STRING,
-                           G_TYPE_INT64,
-                           DBUS_TYPE_G_STRING_STRING_HASHTABLE,
+                           "ItemsAdded",
+                           _mojito_items_get_container_type (),
                            NULL);
   dbus_g_proxy_connect_signal (priv->proxy,
-                               "ItemAdded",
-                               (GCallback)_proxy_item_added_cb,
+                               "ItemsAdded",
+                               (GCallback)_proxy_items_added_cb,
                                object,
                                NULL);
 
   dbus_g_proxy_add_signal (priv->proxy,
-                           "ItemChanged",
-                           G_TYPE_STRING,
-                           G_TYPE_STRING,
-                           G_TYPE_INT64,
-                           G_TYPE_BOXED,
+                           "ItemsChanged",
+                           _mojito_items_get_container_type (),
                            NULL);
   dbus_g_proxy_connect_signal (priv->proxy,
-                               "ItemChanged",
-                               (GCallback)_proxy_item_changed_cb,
+                               "ItemsChanged",
+                               (GCallback)_proxy_items_changed_cb,
                                object,
                                NULL);
 
   dbus_g_proxy_add_signal (priv->proxy,
-                           "ItemRemoved",
-                           G_TYPE_STRING,
-                           G_TYPE_STRING,
+                           "ItemsRemoved",
+                           _mojito_items_removed_get_container_type (),
                            NULL);
   dbus_g_proxy_connect_signal (priv->proxy,
-                               "ItemRemoved",
-                               (GCallback)_proxy_item_removed_cb,
+                               "ItemsRemoved",
+                               (GCallback)_proxy_items_removed_cb,
                                object,
                                NULL);
 }
@@ -312,40 +394,39 @@ mojito_client_view_class_init (MojitoClientViewClass *klass)
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
   g_object_class_install_property (object_class, PROP_OBJECT_PATH, pspec);
 
-  signals[ITEM_ADDED_SIGNAL] =
-    g_signal_new ("item-added",
+  signals[ITEMS_ADDED_SIGNAL] =
+    g_signal_new ("items-added",
                   MOJITO_TYPE_CLIENT_VIEW,
                   G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (MojitoClientViewClass, item_added),
+                  G_STRUCT_OFFSET (MojitoClientViewClass, items_added),
                   NULL,
                   NULL,
                   mojito_marshal_VOID__POINTER,
                   G_TYPE_NONE,
                   1,
                   G_TYPE_POINTER);
-  signals[ITEM_REMOVED_SIGNAL] =
-    g_signal_new ("item-removed",
+  signals[ITEMS_REMOVED_SIGNAL] =
+    g_signal_new ("items-removed",
                   MOJITO_TYPE_CLIENT_VIEW,
                   G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (MojitoClientViewClass, item_removed),
+                  G_STRUCT_OFFSET (MojitoClientViewClass, items_removed),
                   NULL,
                   NULL,
                   mojito_marshal_VOID__POINTER,
                   G_TYPE_NONE,
                   1,
                   G_TYPE_POINTER);
-  signals[ITEM_CHANGED_SIGNAL] =
-    g_signal_new ("item-changed",
+  signals[ITEMS_CHANGED_SIGNAL] =
+    g_signal_new ("items-changed",
                   MOJITO_TYPE_CLIENT_VIEW,
                   G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (MojitoClientViewClass, item_changed),
+                  G_STRUCT_OFFSET (MojitoClientViewClass, items_changed),
                   NULL,
                   NULL,
-                  mojito_marshal_VOID__STRING_STRING,
+                  mojito_marshal_VOID__POINTER,
                   G_TYPE_NONE,
                   1,
                   G_TYPE_POINTER);
-
 }
 
 static void
