@@ -23,7 +23,9 @@ typedef enum {
   MOJITO_DEBUG_VIEWS = 1 << 1,
   MOJITO_DEBUG_ONLINE = 1 << 2,
   MOJITO_DEBUG_ITEM = 1 << 3,
-  MOJITO_DEBUG_TWITTER = 1 << 4
+  MOJITO_DEBUG_TWITTER = 1 << 4,
+  MOJITO_DEBUG_MYSPACE = 1 << 5,
+  MOJITO_DEBUG_LASTFM = 1 << 6
 } MojitoDebugFlags;
 
 extern guint mojito_debug_flags;
