@@ -31,6 +31,7 @@ struct _MojitoItemPrivate {
   MojitoService *service;
   GHashTable *hash;
   time_t cached_date;
+  time_t mtime;
   gint remaining_fetches;
 };
 
@@ -134,6 +135,8 @@ mojito_item_put (MojitoItem *item, const char *key, const char *value)
     g_hash_table_insert (item->priv->hash, (gpointer)g_intern_string (key), g_strdup (value));
   else
     g_hash_table_remove (item->priv->hash, (gpointer)g_intern_string (key));
+
+  mojito_item_touch (item);
 }
 
 void
@@ -146,6 +149,8 @@ mojito_item_take (MojitoItem *item, const char *key, char *value)
     g_hash_table_insert (item->priv->hash, (gpointer)g_intern_string (key), value);
   else
     g_hash_table_remove (item->priv->hash, (gpointer)g_intern_string (key));
+
+  mojito_item_touch (item);
 }
 
 const char *
@@ -262,12 +267,15 @@ mojito_item_pop_pending (MojitoItem *item)
                   mojito_item_get (item, "id"));
     g_object_notify (G_OBJECT (item), "ready");
   }
+
+  mojito_item_touch (item);
 }
 
 
 typedef struct {
   MojitoItem *item;
   const gchar *key;
+  gboolean delays_ready;
 } RequestImageFetchClosure;
 
 static void
@@ -280,7 +288,8 @@ _image_download_cb (const char               *url,
                     closure->key,
                     file);
 
-  mojito_item_pop_pending (closure->item);
+  if (closure->delays_ready)
+    mojito_item_pop_pending (closure->item);
 
   g_object_unref (closure->item);
   g_slice_free (RequestImageFetchClosure, closure);
@@ -288,17 +297,23 @@ _image_download_cb (const char               *url,
 
 void
 mojito_item_request_image_fetch (MojitoItem  *item,
+                                 gboolean     delays_ready,
                                  const gchar *key,
                                  const gchar *url)
 {
   RequestImageFetchClosure *closure;
 
-  mojito_item_push_pending (item);
+  /* If this URL fetch should delay the item being considered ready, or
+   * whether the item is useful without this key.
+   */
+  if (delays_ready)
+    mojito_item_push_pending (item);
 
   closure = g_slice_new0 (RequestImageFetchClosure);
 
   closure->key = g_intern_string (key);
   closure->item = g_object_ref (item);
+  closure->delays_ready = delays_ready;
 
   MOJITO_DEBUG (ITEM, "Scheduling fetch for %s on: %s",
                 url,
@@ -307,3 +322,55 @@ mojito_item_request_image_fetch (MojitoItem  *item,
                                    (ImageDownloadCallback)_image_download_cb,
                                    closure);
 }
+
+/*
+ * Construct a GValueArray from a MojitoItem. We use this to construct the
+ * data types that the wonderful dbus-glib needs to emit the signal
+ */
+GValueArray *
+_mojito_item_to_value_array (MojitoItem *item)
+{
+  GValueArray *value_array;
+  time_t time;
+
+  time = mojito_time_t_from_string (mojito_item_get (item, "date"));
+
+  value_array = g_value_array_new (4);
+
+  value_array = g_value_array_append (value_array, NULL);
+  g_value_init (g_value_array_get_nth (value_array, 0), G_TYPE_STRING);
+  g_value_set_string (g_value_array_get_nth (value_array, 0),
+                      mojito_service_get_name (mojito_item_get_service (item)));
+
+  value_array = g_value_array_append (value_array, NULL);
+  g_value_init (g_value_array_get_nth (value_array, 1), G_TYPE_STRING);
+  g_value_set_string (g_value_array_get_nth (value_array, 1),
+                      mojito_item_get (item, "id"));
+
+  value_array = g_value_array_append (value_array, NULL);
+  g_value_init (g_value_array_get_nth (value_array, 2), G_TYPE_INT64);
+  g_value_set_int64 (g_value_array_get_nth (value_array, 2),
+                     time);
+
+  value_array = g_value_array_append (value_array, NULL);
+  g_value_init (g_value_array_get_nth (value_array, 3),
+                dbus_g_type_get_map ("GHashTable",
+                                     G_TYPE_STRING,
+                                     G_TYPE_STRING));
+  g_value_set_boxed (g_value_array_get_nth (value_array, 3),
+                     mojito_item_peek_hash (item));
+
+  return value_array;
+}
+
+void
+mojito_item_touch (MojitoItem *item)
+{
+  item->priv->mtime = time (NULL);
+}
+
+time_t
+mojito_item_get_mtime (MojitoItem *item)
+{
+  return item->priv->mtime;
+}
