@@ -66,7 +66,10 @@ void mojito_item_put (MojitoItem *item, const char *key, const char *value);
 
 void mojito_item_take (MojitoItem *item, const char *key, char *value);
 
-void mojito_item_request_image_fetch (MojitoItem *item, const gchar *key, const gchar *url);
+void mojito_item_request_image_fetch (MojitoItem  *item,
+                                      gboolean     delays_ready,
+                                      const gchar *key,
+                                      const gchar *url);
 
 const char * mojito_item_get (MojitoItem *item, const char *key);
 
@@ -83,9 +86,14 @@ gboolean mojito_item_get_ready (MojitoItem *item);
 void mojito_item_push_pending (MojitoItem *item);
 void mojito_item_pop_pending (MojitoItem *item);
 
+void mojito_item_touch (MojitoItem *item);
+time_t mojito_item_get_mtime (MojitoItem *item);
 
 /* Convenience function */
-MojitoSet * mojito_item_set_new (void);
+MojitoSet *mojito_item_set_new (void);
+
+/* Useful for emitting the signals */
+GValueArray *_mojito_item_to_value_array (MojitoItem *item);
 
 G_END_DECLS
 
