@@ -38,6 +38,9 @@ G_DEFINE_TYPE_WITH_CODE (MojitoView, mojito_view, G_TYPE_OBJECT,
 /* Refresh every 5 minutes */
 #define REFRESH_TIMEOUT (5 * 60)
 
+/* Wait 200ms between queued recalculates before recalculating */
+#define RECALCULATE_DELAY 200
+
 struct _MojitoViewPrivate {
   MojitoCore *core;
   /* List of MojitoService objects */
@@ -52,6 +55,10 @@ struct _MojitoViewPrivate {
   MojitoSet *current;
   /* Whether we're currently running */
   gboolean running;
+  /* For recalculate queue */
+  guint recalculate_timeout_id;
+  /* The last time we recalculated. Use for changed items. */
+  time_t last_recalculate_time;
 };
 
 enum {
@@ -75,39 +82,6 @@ inc_service_count (GHashTable *hash, MojitoService *service)
   g_hash_table_insert (hash, service, GINT_TO_POINTER (count));
 }
 
-static void
-send_added (gpointer data, gpointer user_data)
-{
-  MojitoItem *item = data;
-  MojitoView *view = user_data;
-  MojitoService *service;
-  time_t time;
-
-  service = mojito_item_get_service (item);
-
-  time = mojito_time_t_from_string (mojito_item_get (item, "date"));
-
-  mojito_view_iface_emit_item_added (view,
-                                     mojito_service_get_name (service),
-                                     mojito_item_get (item, "id"),
-                                     time,
-                                     mojito_item_peek_hash (item));
-}
-
-static void
-send_removed (gpointer data, gpointer user_data)
-{
-  MojitoItem *item = data;
-  MojitoView *view = user_data;
-  MojitoService *service;
-
-  service = mojito_item_get_service (item);
-
-  mojito_view_iface_emit_item_removed (view,
-                                       mojito_service_get_name (service),
-                                       mojito_item_get (item, "id"));
-}
-
 static MojitoSet *
 munge_items (MojitoView *view)
 {
@@ -124,7 +98,6 @@ munge_items (MojitoView *view)
   while (list) {
     item = (MojitoItem *)list->data;
     if (!mojito_core_is_item_banned (priv->core, item)) {
-
       if (mojito_item_get_ready (item)) {
         l = g_list_prepend (l, item);
         MOJITO_DEBUG (VIEWS, "Item ready: %s",
@@ -210,12 +183,109 @@ remove_service (GObject *object, gpointer user_data)
   return mojito_item_get_service (item) == service;
 }
 
+static void
+mojito_view_removed_items (MojitoView *view,
+                           MojitoSet  *removed_items)
+{
+  GValueArray *value_array;
+  GPtrArray *ptr_array;
+  GList *items, *l;
+
+  items = mojito_set_as_list (removed_items);
+  ptr_array = g_ptr_array_new ();
+
+  for (l = items; l; l = l->next)
+  {
+    MojitoItem *item = (MojitoItem *)l->data;
+    value_array = g_value_array_new (2);
+
+    value_array = g_value_array_append (value_array, NULL);
+    g_value_init (g_value_array_get_nth (value_array, 0), G_TYPE_STRING);
+    g_value_set_string (g_value_array_get_nth (value_array, 0),
+                        mojito_service_get_name (mojito_item_get_service (item)));
+
+    value_array = g_value_array_append (value_array, NULL);
+    g_value_init (g_value_array_get_nth (value_array, 1), G_TYPE_STRING);
+    g_value_set_string (g_value_array_get_nth (value_array, 1),
+                        mojito_item_get (item, "id"));
+
+    g_ptr_array_add (ptr_array, value_array);
+  }
+
+  mojito_view_iface_emit_items_removed (view, ptr_array);
+
+  g_ptr_array_foreach (ptr_array, (GFunc)g_value_array_free, NULL);
+  g_ptr_array_free (ptr_array, TRUE);
+}
+
+static void
+mojito_view_added_items (MojitoView *view,
+                         MojitoSet  *added_items)
+{
+  GValueArray *value_array;
+  GPtrArray *ptr_array;
+  GList *items, *l;
+
+  items = mojito_set_as_list (added_items);
+  ptr_array = g_ptr_array_new ();
+
+  for (l = items; l; l = l->next)
+  {
+    MojitoItem *item = (MojitoItem *)l->data;
+    value_array = _mojito_item_to_value_array (item);
+    g_ptr_array_add (ptr_array, value_array);
+  }
+
+  mojito_view_iface_emit_items_added (view,
+                                      ptr_array);
+
+  g_ptr_array_foreach (ptr_array, (GFunc)g_value_array_free, NULL);
+  g_ptr_array_free (ptr_array, TRUE);
+}
+
+
+static void
+mojito_view_changed_items (MojitoView *view,
+                           MojitoSet  *added_items)
+{
+  GValueArray *value_array;
+  GPtrArray *ptr_array;
+  GList *items, *l;
+
+  items = mojito_set_as_list (added_items);
+  ptr_array = g_ptr_array_new ();
+
+  for (l = items; l; l = l->next)
+  {
+    MojitoItem *item = (MojitoItem *)l->data;
+    value_array = _mojito_item_to_value_array (item);
+    g_ptr_array_add (ptr_array, value_array);
+  }
+
+  mojito_view_iface_emit_items_changed (view,
+                                        ptr_array);
+
+  g_ptr_array_foreach (ptr_array, (GFunc)g_value_array_free, NULL);
+  g_ptr_array_free (ptr_array, TRUE);
+}
+
+static gboolean
+_filter_only_changed_cb (MojitoSet  *set,
+                         MojitoItem *item,
+                         MojitoView *view)
+{
+  if (mojito_item_get_mtime (item) >= view->priv->last_recalculate_time)
+    return TRUE;
+  else
+    return FALSE;
+}
+
 void
 mojito_view_recalculate (MojitoView *view)
 {
   MojitoViewPrivate *priv;
   MojitoSet *old_items, *new_items;
-  MojitoSet *removed_items, *added_items;
+  MojitoSet *removed_items, *added_items, *changed_items = NULL;
 
   g_return_if_fail (MOJITO_IS_VIEW (view));
 
@@ -229,14 +299,58 @@ mojito_view_recalculate (MojitoView *view)
   removed_items = mojito_set_difference (old_items, new_items);
   added_items = mojito_set_difference (new_items, old_items);
 
-  mojito_set_foreach (removed_items, (GFunc)send_removed, view);
-  mojito_set_foreach (added_items, (GFunc)send_added, view);
+  if (priv->last_recalculate_time != 0)
+  {
+    changed_items = mojito_set_filter (new_items,
+                                       (MojitoSetFilterFunc)_filter_only_changed_cb,
+                                       view);
+  }
+
+  if (!mojito_set_is_empty (removed_items))
+    mojito_view_removed_items (view, removed_items);
+
+  if (!mojito_set_is_empty (added_items))
+    mojito_view_added_items (view, added_items);
+
+  if (changed_items && !mojito_set_is_empty (changed_items))
+    mojito_view_changed_items (view, changed_items);
 
   mojito_set_unref (removed_items);
   mojito_set_unref (added_items);
 
+  if (changed_items)
+    mojito_set_unref (changed_items);
+
   mojito_set_unref (priv->current);
   priv->current = new_items;
+  priv->last_recalculate_time = time (NULL);
+}
+
+static gboolean
+recalculate_timeout_cb (MojitoView *view)
+{
+  view->priv->recalculate_timeout_id = 0;
+  mojito_view_recalculate (view);
+
+  return FALSE;
+}
+
+static void
+mojito_view_queue_recalculate (MojitoView *view)
+{
+  MojitoViewPrivate *priv;
+
+  priv = view->priv;
+
+  if (priv->recalculate_timeout_id)
+  {
+    g_source_remove (priv->recalculate_timeout_id);
+  }
+
+  priv->recalculate_timeout_id =
+    g_timeout_add (RECALCULATE_DELAY,
+                   (GSourceFunc)recalculate_timeout_cb,
+                   view);
 }
 
 static void
@@ -248,7 +362,7 @@ _item_ready_notify_cb (MojitoItem *item,
   if (mojito_item_get_ready (item)) {
     MOJITO_DEBUG (VIEWS, "Item became ready: %s.",
                   mojito_item_get (item, "id"));
-    mojito_view_recalculate (view);
+    mojito_view_queue_recalculate (view);
     g_signal_handlers_disconnect_by_func (item,
                                           _item_ready_notify_cb,
                                           view);
@@ -476,6 +590,12 @@ mojito_view_dispose (GObject *object)
   MojitoView *view = MOJITO_VIEW (object);
   MojitoViewPrivate *priv = view->priv;
 
+  if (priv->recalculate_timeout_id)
+  {
+    g_source_remove (priv->recalculate_timeout_id);
+    priv->recalculate_timeout_id = 0;
+  }
+
   if (priv->core) {
     g_object_unref (priv->core);
     priv->core = NULL;
