@@ -34,7 +34,25 @@
 #include <rest/rest-xml-parser.h>
 #include <libsoup/soup.h>
 
-G_DEFINE_TYPE (MojitoServiceTwitter, mojito_service_twitter, MOJITO_TYPE_SERVICE)
+#include <interfaces/mojito-query-ginterface.h>
+#include <interfaces/mojito-avatar-ginterface.h>
+#include <interfaces/mojito-status-update-ginterface.h>
+
+#include "twitter-item-view.h"
+
+static void query_iface_init (gpointer g_iface, gpointer iface_data);
+static void avatar_iface_init (gpointer g_iface, gpointer iface_data);
+static void status_update_iface_init (gpointer g_iface, gpointer iface_data);
+
+G_DEFINE_TYPE_WITH_CODE (MojitoServiceTwitter,
+                         mojito_service_twitter,
+                         MOJITO_TYPE_SERVICE,
+                         G_IMPLEMENT_INTERFACE (MOJITO_TYPE_QUERY_IFACE,
+                                                query_iface_init)
+                         G_IMPLEMENT_INTERFACE (MOJITO_TYPE_AVATAR_IFACE,
+                                                avatar_iface_init)
+                         G_IMPLEMENT_INTERFACE (MOJITO_TYPE_STATUS_UPDATE_IFACE,
+                                                status_update_iface_init));
 
 #define GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MOJITO_TYPE_SERVICE_TWITTER, MojitoServiceTwitterPrivate))
@@ -44,7 +62,8 @@ G_DEFINE_TYPE (MojitoServiceTwitter, mojito_service_twitter, MOJITO_TYPE_SERVICE
 struct _MojitoServiceTwitterPrivate {
   enum {
     OWN,
-    FRIENDS
+    FRIENDS,
+    BOTH
   } type;
   gboolean running;
   RestProxy *proxy;
@@ -171,11 +190,14 @@ make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
 
   u_node = rest_xml_node_find (node, "user");
 
-  user_id = rest_xml_node_find (u_node, "screen_name")->content;
+  user_id = rest_xml_node_find (u_node, "id")->content;
 
-  /* For friend feeds, ignore our own tweets */
-  if (priv->type == FRIENDS && g_str_equal (user_id, priv->user_id))
+  /* For friend only feeds, ignore our own tweets */
+  if (priv->type == FRIENDS &&
+      user_id && g_str_equal (user_id, priv->user_id))
+  {
     return NULL;
+  }
 
   item = mojito_item_new ();
   mojito_item_set_service (item, (MojitoService *)twitter);
@@ -197,7 +219,7 @@ make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
     /* Construct the thumbnail URL and download the image */
     twitpic_id = g_match_info_fetch (match_info, 1);
     url = g_strconcat ("http://twitpic.com/show/thumb/", twitpic_id, NULL);
-    mojito_item_request_image_fetch (item, "thumbnail", url);
+    mojito_item_request_image_fetch (item, FALSE, "thumbnail", url);
     g_free (url);
 
     /* Remove the URL from the tweet and use that as the title */
@@ -214,9 +236,10 @@ make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
     mojito_item_take (item, "url", url);
 
     g_free (twitpic_id);
-  } else {
-    mojito_item_put (item, "content", content);
   }
+
+  mojito_item_put (item, "content", content);
+
   g_match_info_free (match_info);
 
   date = rest_xml_node_find (node, "created_at")->content;
@@ -228,7 +251,7 @@ make_item (MojitoServiceTwitter *twitter, RestXmlNode *node)
 
   n = rest_xml_node_find (u_node, "profile_image_url");
   if (n && n->content)
-    mojito_item_request_image_fetch (item, "authoricon", n->content);
+    mojito_item_request_image_fetch (item, FALSE, "authoricon", n->content);
 
 
   return item;
@@ -290,9 +313,11 @@ get_status_updates (MojitoServiceTwitter *twitter)
     rest_proxy_call_set_function (call, "statuses/user_timeline.xml");
     break;
   case FRIENDS:
+  case BOTH:
     rest_proxy_call_set_function (call, "statuses/friends_timeline.xml");
     break;
   }
+
   rest_proxy_call_async (call, tweets_cb, (GObject*)twitter, NULL, NULL);
 }
 
@@ -569,10 +594,12 @@ mojito_service_twitter_constructed (GObject *object)
 
   priv = twitter->priv = GET_PRIVATE (twitter);
 
-  if (mojito_service_get_param ((MojitoService*)twitter, "own")) {
+  if (mojito_service_get_param ((MojitoService *)twitter, "own")) {
     priv->type = OWN;
-  } else {
+  } else if (mojito_service_get_param ((MojitoService *)twitter, "friends")){
     priv->type = FRIENDS;
+  } else {
+    priv->type = BOTH;
   }
 
   priv->twitpic_re = g_regex_new ("http://twitpic.com/([A-Za-z0-9]+)", 0, 0, NULL);
@@ -670,3 +697,126 @@ mojito_service_twitter_init (MojitoServiceTwitter *self)
 {
   self->priv = GET_PRIVATE (self);
 }
+
+/* Query interface */
+
+static void
+_twitter_query_open_view (MojitoQueryIface      *self,
+                          GHashTable            *params,
+                          DBusGMethodInvocation *context)
+{
+  MojitoServiceTwitterPrivate *priv = GET_PRIVATE (self);
+  MojitoItemView *item_view;
+  const gchar *object_path;
+
+  item_view = g_object_new (MOJITO_TYPE_TWITTER_ITEM_VIEW,
+                            "proxy", priv->proxy,
+                            "service", self,
+                            NULL);
+
+  object_path = mojito_item_view_get_object_path (item_view);
+  mojito_query_iface_return_from_open_view (context,
+                                            object_path);
+}
+
+static void
+query_iface_init (gpointer g_iface,
+                  gpointer iface_data)
+{
+  MojitoQueryIfaceClass *klass = (MojitoQueryIfaceClass*)g_iface;
+
+  mojito_query_iface_implement_open_view (klass,
+                                          _twitter_query_open_view);
+}
+
+/* Avatar interface */
+
+static void
+_requested_avatar_downloaded_cb (const gchar *uri,
+                                 gchar       *local_path,
+                                 gpointer     userdata)
+{
+  MojitoService *service = MOJITO_SERVICE (userdata);
+
+  mojito_avatar_iface_emit_avatar_retrieved (service, local_path);
+  g_free (local_path);
+}
+
+static void
+_twitter_avatar_request_avatar (MojitoAvatarIface     *self,
+                                DBusGMethodInvocation *context)
+{
+  MojitoServiceTwitterPrivate *priv = GET_PRIVATE (self);
+
+  if (priv->image_url) {
+    mojito_web_download_image_async (priv->image_url,
+                                     _requested_avatar_downloaded_cb,
+                                     self);
+  }
+
+  mojito_avatar_iface_return_from_request_avatar (context);
+}
+
+static void
+avatar_iface_init (gpointer g_iface,
+                   gpointer iface_data)
+{
+  MojitoAvatarIfaceClass *klass = (MojitoAvatarIfaceClass*)g_iface;
+
+  mojito_avatar_iface_implement_request_avatar (klass,
+                                                _twitter_avatar_request_avatar);
+}
+
+/* Status Update interface */
+
+static void
+_update_status_cb (RestProxyCall *call,
+                   const GError  *error,
+                   GObject       *weak_object,
+                   gpointer       userdata)
+{
+  if (error)
+  {
+    g_critical (G_STRLOC ": Error updating status: %s",
+                error->message);
+    mojito_status_update_iface_emit_status_updated (weak_object, FALSE);
+  } else {
+    MOJITO_DEBUG (TWITTER, G_STRLOC ": Status updated.");
+    mojito_status_update_iface_emit_status_updated (weak_object, FALSE);
+  }
+}
+
+static void
+_twitter_status_update_update_status (MojitoStatusUpdateIface *self,
+                                      const gchar             *msg,
+                                      DBusGMethodInvocation   *context)
+{
+  MojitoServiceTwitter *twitter = MOJITO_SERVICE_TWITTER (self);
+  MojitoServiceTwitterPrivate *priv = twitter->priv;
+  RestProxyCall *call;
+
+  if (!priv->user_id)
+    return;
+
+  call = rest_proxy_new_call (priv->proxy);
+  rest_proxy_call_set_method (call, "POST");
+  rest_proxy_call_set_function (call, "statuses/update.xml");
+
+  rest_proxy_call_add_params (call,
+                              "status", msg,
+                              NULL);
+
+  rest_proxy_call_async (call, _update_status_cb, (GObject *)self, NULL, NULL);
+  mojito_status_update_iface_return_from_update_status (context);
+}
+
+static void
+status_update_iface_init (gpointer g_iface,
+                          gpointer iface_data)
+{
+  MojitoStatusUpdateIfaceClass *klass = (MojitoStatusUpdateIfaceClass*)g_iface;
+
+  mojito_status_update_iface_implement_update_status (klass,
+                                                      _twitter_status_update_update_status);
+}
+
